// Generated by CoffeeScript 1.6.3
/*
# Plugin for the Americano web framework that take care of your Model
# by providing helpers to build them (it wraps jugglingdb!) and asking you
# for writing your requests in a single and clean file.
#
# This plugin has only sense for Cozy applications.
*/


(function() {
  var Schema, async, db, settings, _getRequestCreator, _loadModels, _loadRequestCreators;

  async = require('async');

  _loadModels = function(root, requests) {
    var docRequests, docType, models;
    models = [];
    for (docType in requests) {
      docRequests = requests[docType];
      models[docType] = require("" + root + "/server/models/" + docType);
    }
    return models;
  };

  _getRequestCreator = function(root, models, docType, requestName, request) {
    return function(cb) {
      console.info("[INFO] " + docType + " - " + requestName + " request creation...");
      return models[docType].defineRequest(requestName, request, function(err) {
        if (err) {
          console.log("[ERROR]... fail");
        } else {
          console.info("[INFO] ... ok");
        }
        return cb(err);
      });
    };
  };

  _loadRequestCreators = function(root, models, requests) {
    var docRequest, docRequests, docType, requestName, requestsToSave;
    requestsToSave = [];
    for (docType in requests) {
      docRequests = requests[docType];
      for (requestName in docRequests) {
        docRequest = docRequests[requestName];
        requestsToSave.push(_getRequestCreator(root, models, docType, requestName, docRequest));
      }
    }
    return requestsToSave;
  };

  module.exports.configure = function(root, app, callback) {
    var err, models, requests, requestsToSave;
    try {
      requests = require("" + root + "/server/models/requests");
    } catch (_error) {
      err = _error;
      console.log("[ERROR] failed to load requests file");
      callback(err);
    }
    models = _loadModels(root, requests);
    requestsToSave = _loadRequestCreators(root, models, requests);
    return async.series(requestsToSave, function(err) {
      if (err && err.code !== 'EEXIST') {
        console.log("[ERROR] A request creation failed, abandon.");
        console.log(err);
        if (callback != null) {
          return callback(err);
        }
      } else {
        console.info("[INFO] All requests have been created");
        if (callback != null) {
          return callback();
        }
      }
    });
  };

  Schema = require('jugglingdb').Schema;

  settings = {
    url: 'http://localhost:9101/'
  };

  module.exports.db = db = new Schema('cozy-adapter', settings);

  module.exports.getModel = function(name, fields) {
    var model;
    fields.id = String;
    model = db.define(name, fields);
    return model;
  };

  module.exports.defaultRequests = {
    all: function(doc) {
      return emit(doc._id, doc);
    }
  };

}).call(this);
